package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.sys_area.GetSysAreaInput;
import pps.cloud.system.service.data.sys_area.GetSysAreaOutput;
import pps.core.base.entity.BaseWindTowerEnt;
import pps.core.base.entity.BaseWindTowerView;
import pps.core.base.entity.BaseWindTurbineEnt;
import pps.core.base.entity.BaseWindTurbineView;
import pps.core.base.mapper.BaseWindTowerMapper;
import pps.core.base.mapper.BaseWindTowerViewMapper;
import pps.core.base.mapper.BaseWindTurbineMapper;
import pps.core.base.mapper.BaseWindTurbineViewMapper;
import pps.core.base.service.data.base_wind_tower.BaseWindTowerInput;
import pps.core.base.service.data.base_wind_turbine.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 基础风机电站
 *
 * @author ZWT
 * @date 2024/08/28
 */
@XText("基础风机电站")
@XService
public class BaseWindTurbineService {

    /**
     * 详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetBaseWindTurbineOutput }>
     */
    @XText("详情")
    @XApiPost
    public XSingleResult<GetBaseWindTurbineOutput> getBaseWindTurbine(XContext context, GetBaseWindTurbineInput input) {
        //查主表信息
        BaseWindTurbineMapper mapper = context.getBean(BaseWindTurbineMapper.class);
        BaseWindTurbineEnt entity = this.getOne(mapper, input.getId());
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        //查关联信息
        BaseWindTowerMapper towerMapper = context.getBean(BaseWindTowerMapper.class);
        List<BaseWindTowerEnt> towerEntList = towerMapper.selectList(new LambdaQueryWrapper<BaseWindTowerEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BaseWindTowerEnt::getStationId, entity.getId())
        );
        //封装返回信息
        GetBaseWindTurbineOutput output = XCopyUtils.copyNewObject(entity, GetBaseWindTurbineOutput.class);
        output.setWindTowerList(XCopyUtils.copyNewList(towerEntList, BaseWindTowerInput.class));
        output.setAreaName(Optional.ofNullable(this.getSysAreaInfo(context, GetSysAreaInput.builder()
                        .id(entity.getAreaCode())
                        .build()))
                .map(GetSysAreaOutput::getName)
                .orElse(null));
        //todo 后面加逻辑
        output.setAccuracy("66.6");
        return XSingleResult.success(output);
    }

    /**
     * 新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("新增")
    @XApiPost
    public XServiceResult createBaseWindTurbine(XContext context, CreateBaseWindTurbineInput input) {
        return XTransactionHelper.begin(context, () -> {
            PpsUserSession session = getSession(context);
            //封装主表信息
            BaseWindTurbineEnt entity = XCopyUtils.copyNewObject(input, BaseWindTurbineEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            BaseWindTurbineMapper mapper = context.getBean(BaseWindTurbineMapper.class);
            mapper.insert(entity);
            //封装子表信息
            this.batchInsert(context, input.getWindTowerList(), session, entity.getId());
            return XServiceResult.OK;
        });
    }

    /**
     * 修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("修改")
    @XApiPost
    public XServiceResult updateBaseWindTurbine(XContext context, UpdateBaseWindTurbineInput input) {
        return XTransactionHelper.begin(context, () -> {
            //更新主表信息
            BaseWindTurbineMapper mapper = context.getBean(BaseWindTurbineMapper.class);
            BaseWindTurbineEnt entity = this.getOne(mapper, input.getId());
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            PpsUserSession session = getSession(context);
            XCopyUtils.copyObject(input, entity);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            //删除关联信息
            BaseWindTowerMapper towerMapper = context.getBean(BaseWindTowerMapper.class);
            towerMapper.delete(new LambdaQueryWrapper<BaseWindTowerEnt>()
                    .eq(BaseWindTowerEnt::getStationId, entity.getId())
            );
            //封装子表信息
            this.batchInsert(context, input.getWindTowerList(), session, entity.getId());
            return XServiceResult.OK;
        });
    }

    /**
     * 删除
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("删除")
    @XApiPost
    public XServiceResult deleteBaseWindTurbine(XContext context, GetBaseWindTurbineInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseWindTurbineMapper mapper = context.getBean(BaseWindTurbineMapper.class);
            BaseWindTurbineEnt entity = this.getOne(mapper, input.getId());
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //更新主表
            PpsUserSession session = getSession(context);
            BaseUtils.setBaseModelDefault(entity, session);
            entity.setIsDeleted(BusinessConstant.ZERO);
            mapper.updateById(entity);
            //更新关联信息
            BaseWindTowerMapper towerMapper = context.getBean(BaseWindTowerMapper.class);
            towerMapper.update(null, new LambdaUpdateWrapper<BaseWindTowerEnt>()
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                    .eq(BaseWindTowerEnt::getStationId, entity.getId())
            );
            return XServiceResult.OK;
        });
    }

    /**
     * 分页查询
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult }<{@link QueryBaseWindTurbineViewOutput }>
     */
    @XText("分页查询")
    @XApiPost
    public XPageResult<QueryBaseWindTurbineViewOutput> queryBaseWindTurbineView(XContext context, QueryBaseWindTurbineViewInput input) {
        BaseWindTurbineViewMapper mapper = context.getBean(BaseWindTurbineViewMapper.class);
        BaseWindTurbineView record = XCopyUtils.copyNewObject(input, BaseWindTurbineView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setOuList(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        List<BaseWindTurbineView> list = mapper.selectList(record);
        PageInfo<BaseWindTurbineView> pageInfo = new PageInfo<>(list);
        List<QueryBaseWindTurbineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseWindTurbineViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 获取会话
     *
     * @param context 上下文
     * @return {@link PpsUserSession }
     */
    private PpsUserSession getSession(XContext context) {
        return context.getSession(PpsUserSession.class);
    }

    /**
     * 查询一个
     *
     * @param mapper 映射器
     * @param id     身份证件
     * @return {@link BaseWindTurbineEnt }
     */
    private BaseWindTurbineEnt getOne(BaseWindTurbineMapper mapper, String id) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseWindTurbineEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BaseModel::getId, id)
                .last(BusinessConstant.LAST_LIMIT)
        );
    }

    /**
     * 批量插入
     *
     * @param context       上下文
     * @param windTowerList 风塔列表
     * @param session       阶段
     * @param stationId     站点id
     */
    private void batchInsert(XContext context, List<BaseWindTowerInput> windTowerList, PpsUserSession session, String stationId) {
        if (CollUtil.isNotEmpty(windTowerList)) {
            BaseWindTowerView view;
            List<BaseWindTowerView> windTowerViewList = new ArrayList<>(windTowerList.size());
            for (BaseWindTowerInput towerInput : windTowerList) {
                view = new BaseWindTowerView();
                BaseUtils.setBaseModelDefault(view, session);
                view.setStationId(stationId);
                view.setTowerId(towerInput.getTowerId());
                view.setTowerName(towerInput.getTowerName());
                view.setId(UUID.randomUUID(true).toString(true));
                windTowerViewList.add(view);
            }
            BaseWindTowerViewMapper viewMapper = context.getBean(BaseWindTowerViewMapper.class);
            viewMapper.batchInsert(windTowerViewList);
        }
    }

    /**
     * 获取系统区域信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link GetSysAreaOutput}
     */
    private GetSysAreaOutput getSysAreaInfo(XContext context, GetSysAreaInput input) {
        SystemAreaService systemAreaService = context.getBean(SystemAreaService.class);
        XSingleResult<GetSysAreaOutput> singleResult = systemAreaService.getSysAreaByParam(context, input);
        singleResult.throwIfFail();
        return singleResult.getResult();
    }
}
