package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.excel.EasyExcelFactory;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.excel.util.EasyExcelUtil;
import pps.core.common.excel.util.ExcelStyleTool;
import pps.core.space.enums.BusinessError;
import pps.core.space.service.data.base_excel.ExcelSpaceDurationOutput;
import pps.core.space.service.data.base_excel.ExcelSpaceOptimizeTemplate;
import pps.core.space.service.data.base_excel.ExcelSpaceWellheadOutput;
import pps.core.space.service.data.base_excel.ImportExcelInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiUpload;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * excel导入导出
 *
 * @author ZWT
 * @date 2024/05/14 13:44
 */
@XText("Excel导入导出")
@XService
public class ExcelService {

    /**
     * 编辑制度导出
     *
     * @param context 上下文
     * @return {@link XFileResult }
     */
    @XText("编辑制度导出")
    @XApiGet
    public XFileResult exportEditorialSystem(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/编辑制度模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(Collections.emptyList(),
                        EasyExcelUtil.writeSelectedSheet(ExcelSpaceOptimizeTemplate.class, 0, "编辑制度")
                ).finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 编辑制度导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("编辑制度导入")
    @XApiUpload
    public XSingleResult<ExcelSpaceWellheadOutput> importPriceStrategy(XContext context, ImportExcelInput input) {
        List<ExcelSpaceOptimizeTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelSpaceOptimizeTemplate.class,
                null, null);
        if (CollUtil.isEmpty(list)) {
            return XSingleResult.error(context, BusinessError.UndiscoveredData);
        }
        //设置行号并转换开始/结束时间
        List<Integer> errorRowList = new ArrayList<>(list.size());
        int rowNum = 2;
        for (ExcelSpaceOptimizeTemplate template : list) {
            try {
                //转换开始时间和结束时间(开始时间不能为24:00)
                template.setStart(DateTime.of(template.getStartTime(), BusinessConstant.MINUTES_FORMAT));
                if (CharSequenceUtil.equals(template.getEndTime(), BusinessConstant.END_OF_DAY_TIME)) {
                    template.setEnd(BusinessConstant.DATE_FLAG);
                } else {
                    template.setEnd(DateTime.of(template.getEndTime(), BusinessConstant.MINUTES_FORMAT));
                }
                template.setDuration(DateUtil.between(template.getStart(), template.getEnd(), DateUnit.MINUTE));
            } catch (Exception e) {
                errorRowList.add(rowNum);
            }
            template.setRowNum(rowNum++);
        }
        if (CollUtil.isNotEmpty(errorRowList)) {
            throw new XServiceException("第[" + CollUtil.join(errorRowList, ",") + "]行,开始或结束时间输入错误!");
        }
        //开关井时间校验
        if (list.size() > 1) {
            //排序
            list.sort(Comparator.comparing(ExcelSpaceOptimizeTemplate::getStartTime));
            StringBuilder stringBuilder = new StringBuilder();
            ExcelSpaceOptimizeTemplate start = list.get(0);
            DateTime firstEnd = start.getEnd();
            if (start.getStart().compareTo(firstEnd) >= 0) {
                stringBuilder.append("第[").append(start.getRowNum()).append("]行,开始时间大于等于结束时间;");
            }
            for (int i = 1; i < list.size(); i++) {
                ExcelSpaceOptimizeTemplate end = list.get(i);
                DateTime secondStart = end.getStart();
                DateTime secondEnd = end.getEnd();
                if (secondStart.compareTo(secondEnd) >= 0) {
                    stringBuilder.append("第[").append(start.getRowNum()).append("]行,开始时间大于等于结束时间;");
                } else if (firstEnd.compareTo(secondStart) > 0) {
                    stringBuilder.append("第[").append(start.getRowNum()).append("]行,结束时间大于第[").append(end.getRowNum()).append("]行开始时间;");
                }
                start = end;
                firstEnd = secondEnd;
            }
            if (stringBuilder.length() > 0) {
                throw new XServiceException(stringBuilder.toString());
            }
        }
        //计算随动时间
        List<ExcelSpaceDurationOutput> durationList = new ArrayList<>(list.size());
        long runDuration = 0;
        long maxOpen = 0;
        long minOpen = -1;
        long maxClose = 0;
        long minClose = -1;
        for (int i = 0; i < list.size(); i++) {
            ExcelSpaceOptimizeTemplate template = list.get(i);
            if (maxOpen < template.getDuration()) {
                maxOpen = template.getDuration();
            }
            if (minOpen > template.getDuration() || minOpen == -1) {
                minOpen = template.getDuration();
            }
            if (i + 1 < list.size()) {
                long between = DateUtil.between(template.getEnd(), list.get(i + 1).getStart(), DateUnit.MINUTE);
                if (maxClose < between) {
                    maxClose = between;
                }
                if (minClose > between || minClose == -1) {
                    minClose = between;
                }
            }
            runDuration += template.getDuration();
            durationList.add(
                    ExcelSpaceDurationOutput.builder()
                            .openWellTime(template.getStartTime())
                            .closeWellTime(template.getEndTime())
                            .build()
            );
        }
        //分钟转小时
        return XSingleResult.success(ExcelSpaceWellheadOutput.builder()
                .durationList(durationList)
                .runDuration(this.conversionHour(runDuration))
                .maxOpen(this.conversionHour(maxOpen))
                .minOpen(this.conversionHour(minOpen))
                .maxClose(this.conversionHour(maxClose))
                .minClose(this.conversionHour(minClose))
                .build());
    }

    /*---------------------------- private ----------------------------*/

    /**
     * 转换小时
     *
     * @param duration 期间
     * @return {@link BigDecimal }
     */
    private BigDecimal conversionHour(long duration) {
        if (0 == duration) {
            return BigDecimal.ZERO;
        }
        return new BigDecimal(duration).divide(BusinessConstant.BIG_DECIMAL_60, 1, RoundingMode.HALF_UP);
    }
}